@@ -13,7 +13,6 @@ from oslo_concurrency import processutils
 from shakenfist import config
 from shakenfist import db
 from shakenfist import dhcp
-from shakenfist import db
 from shakenfist import util
 
 
@@ -27,6 +26,10 @@ def from_db(uuid):
     if not dbnet:
         return None
 
+    if dbnet['state'] == 'deleted':
+        LOG.info('network(%s) net.from_db() network is state=deleted' % uuid)
+        return None
+
     return Network(uuid=dbnet['uuid'],
                    vxlan_id=dbnet['vxid'],
                    provide_dhcp=dbnet['provide_dhcp'],
@@ -274,6 +277,9 @@ class Network(object):
                         db.persist_ipmanager('floating', ipm.save())
 
     def update_dhcp(self):
+        if not self.provide_dhcp:
+            return
+
         if config.parsed.get('NODE_IP') == config.parsed.get('NETWORK_NODE_IP'):
             self.ensure_mesh()
             subst = self.subst_dict()
