@@ -46,12 +46,22 @@ class Scheduler(object):
         return True
 
     def _has_sufficient_ram(self, memory, node):
-        max_ram = (self.metrics[node].get('memory_max', 0) *
-                   config.parsed.get('RAM_OVERCOMMIT_RATIO'))
-        current_ram = self.metrics[node].get(
-            'memory_total_instance_actual_memory', 0)
-        if current_ram + memory > max_ram:
+        # There are two things to track here... We must always have RAM_SYSTEM_RESERVATION
+        # gb of RAM for operating system tasks -- assume there is no overlap with existing
+        # VMs when checking this. Note as well that metrics are in MB...
+        available = (self.metrics[node].get('memory_available', 0) -
+                     (config.parsed.get('RAM_SYSTEM_RESERVATION') * 1024))
+        if available - memory < 0.0:
             return False
+
+        # ...Secondly, if we're using KSM and over committing memory, we shouldn't
+        # overcommit more than by RAM_OVERCOMMIT_RATIO
+        instance_memory = (self.metrics[node].get('memory_total_instance_actual', 0) +
+                           memory)
+        if (instance_memory / self.metrics[node].get('memory_max', 0) >
+                config.parsed.get('RAM_OVERCOMMIT_RATIO')):
+            return False
+
         return True
 
     def _has_sufficient_disk(self, instance, node):
@@ -192,9 +202,9 @@ class Scheduler(object):
                 candidates = self._find_most_matching_networks(
                     requested_networks, candidates)
                 LOG.info('Scheduling %s, %s have most matching networks'
-                            % (instance, candidates))
+                         % (instance, candidates))
                 db.add_event('instance', instance.db_entry['uuid'],
-                                'schedule', 'Have most matching networks', None, str(candidates))
+                             'schedule', 'Have most matching networks', None, str(candidates))
 
             # What nodes have the base image already?
             requested_images = []
@@ -214,7 +224,7 @@ class Scheduler(object):
             if len(candidates) > 1 and net_node['fqdn'] in candidates:
                 candidates.remove(net_node['fqdn'])
                 LOG.info('Scheduling %s, %s are non-network nodes'
-                            % (instance, candidates))
+                         % (instance, candidates))
                 db.add_event('instance', instance.db_entry['uuid'],
                              'schedule', 'Are non-network nodes', None, str(candidates))
 
